@@ -2,7 +2,7 @@ package Finance::Budget;
 
 use strict;
 use warnings;
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 {   use Carp;
     use Text::CSV;
     use Date::Manip;
@@ -18,6 +18,7 @@ sub new
 
     my %self = (
         days              => 365,
+        currency          => '$',
         date_format       => '%m/%d/%Y',
         markup_callback   => sub { return $_[0]->{string} },
         opening_balance   => 0,
@@ -85,6 +86,8 @@ sub new
     croak 'there must be 1+ categorizer for each transaction type'
         if @{ $self{transaction_types} } < keys %{ $self{categorizer} };
 
+    $self{opening_balance} *= 100; # dollars to cents
+
     _set_base_dates( \%self );
 
     _build_transactions( \%self );
@@ -92,6 +95,12 @@ sub new
     return bless \%self, $class;
 }
 
+sub opening_balance
+{
+    my ($self) = @_;
+    return _as_dollars( $self->{opening_balance} );
+}
+
 sub next
 {
     my ($self) = @_;
@@ -102,6 +111,20 @@ sub next
     return shift @{ $self->{transactions} };
 }
 
+sub get_chokepoints
+{
+    my ($self) = @_;
+
+    croak "no chokepoints encountered"
+        if not exists $self->{chokepoints};
+
+    return Finance::Budget::Chokepoints->new(
+        {   chokepoints => $self->{chokepoints},
+            markup      => $self->{markup_callback},
+        }
+    );
+}
+
 sub _set_base_dates
 {
     my ($self) = @_;
@@ -210,14 +233,25 @@ sub _set_base_dates
 sub _build_transactions {
     my ($self) = @_;
 
-    my %events_for;
-    my @transactions;
-    my $balance = 100 * $self->{opening_balance};
     my $width = 0;
+    my $balance = $self->{opening_balance};
+    my ( %events_for, @transactions, @chokepoints );
 
     my $start_date = ParseDate('today');
     my $end_date   = DateCalc('today', sprintf '+%dD', $self->{days});
 
+    my ($major_income)
+        = map { $_->{category} }
+        sort  { $b->{amount} <=> $a->{amount} }
+        @{ $self->{transaction_types} };
+
+    my ($major_payment)
+        = map { $_->{category} }
+        sort  { $a->{amount} <=> $b->{amount} }
+        @{ $self->{transaction_types} };
+
+    my $major_payment_hit = 0;
+
     for my $type_hr (@{ $self->{transaction_types} })
     {
         my $category   = $type_hr->{category};
@@ -253,6 +287,28 @@ sub _build_transactions {
 
         for my $event_hr (@events)
         {
+            if ( $event_hr->{category} eq $major_income )
+            {
+                if ( $major_payment_hit )
+                {
+                    my $past_hr = $transactions[-1]->{event};
+
+                    push @chokepoints,
+                        {   date        => $past_hr->{date},
+                            date_str    => $past_hr->{date_str},
+                            cents       => $balance,
+                            balance     => $balance,
+                            balance_str => _as_dollars($balance),
+                        };
+                }
+
+                $major_payment_hit = 0;
+            }
+            elsif ( $event_hr->{category} eq $major_payment )
+            {
+                $major_payment_hit = 1;
+            }
+
             $balance += $event_hr->{cents};
 
             push @transactions,
@@ -267,69 +323,12 @@ sub _build_transactions {
         }
     }
 
+    $self->{chokepoints}  = \@chokepoints;
     $self->{transactions} = \@transactions;
 
     return;
 }
 
-package Finance::Budget::Transaction;
-
-use strict;
-use warnings;
-use overload (
-    q{""} => \&_stringify,
-);
-
-sub new {
-    my ($class, $conf_hr) = @_;
-
-    $conf_hr->{width} += 1;
-
-    $conf_hr->{event}->{date_str}
-        = $conf_hr->{event}->{date}->printf($conf_hr->{date_format});
-
-    $conf_hr->{event}->{title}
-        = sprintf "% $conf_hr->{width}s", $conf_hr->{event}->{category};
-
-    $conf_hr->{event}->{amount} = _as_dollars( $conf_hr->{event}->{cents} );
-
-    $conf_hr->{event}->{balance_str} = _as_dollars( $conf_hr->{balance} );
-
-    $conf_hr->{event}->{string} = join ' ',
-        @{ $conf_hr->{event} }{qw( date_str title amount balance_str )};
-
-    return bless $conf_hr, $class;
-}
-
-sub get_date {
-    my ($self) = @_;
-    return $self->{event}->{date};
-}
-
-sub get_category {
-    my ($self) = @_;
-    return $self->{event}->{category};
-}
-
-sub get_amount {
-    my ($self) = @_;
-    return $self->{event}->{amount};
-}
-
-sub get_balance {
-    my ($self) = @_;
-    return $self->{balance};
-}
-
-sub _stringify {
-    my ($self) = @_;
-
-    return $self->{markup}->( $self->{event} )
-        if $self->{markup};
-
-    return $self->{event}->{string};
-}
-
 sub _as_dollars {
     my ($cents) = @_;
 
@@ -351,6 +350,140 @@ sub _as_dollars {
     return sprintf '% 10s', "$sign\$$dollars";
 }
 
+package Finance::Budget::Transaction;
+
+use strict;
+use warnings;
+use overload (
+    q{""} => \&_stringify,
+);
+
+sub new {
+    my ($class, $conf_hr) = @_;
+
+    $conf_hr->{width} += 1;
+
+    $conf_hr->{event}->{date_str}
+        = $conf_hr->{event}->{date}->printf($conf_hr->{date_format});
+
+    $conf_hr->{event}->{title}
+        = sprintf "% $conf_hr->{width}s", $conf_hr->{event}->{category};
+
+    $conf_hr->{event}->{amount}
+        = Finance::Budget::_as_dollars( $conf_hr->{event}->{cents} );
+
+    $conf_hr->{event}->{balance} = $conf_hr->{balance};
+
+    $conf_hr->{event}->{balance_str}
+        = Finance::Budget::_as_dollars( $conf_hr->{balance} );
+
+    $conf_hr->{event}->{string} = join ' ',
+        @{ $conf_hr->{event} }{qw( date_str title amount balance_str )};
+
+    return bless $conf_hr, $class;
+}
+
+sub get_date {
+    my ($self) = @_;
+    return $self->{event}->{date_str};
+}
+
+sub get_category {
+    my ($self) = @_;
+    return $self->{event}->{category};
+}
+
+sub get_amount {
+    my ($self) = @_;
+    return $self->{event}->{amount};
+}
+
+sub get_balance {
+    my ($self) = @_;
+    return $self->{balance_str};
+}
+
+sub _stringify {
+    my ($self) = @_;
+
+    return $self->{markup}->( $self->{event} )
+        if $self->{markup};
+
+    return $self->{event}->{string};
+}
+
+package Finance::Budget::Chokepoints;
+
+use strict;
+use warnings;
+use Carp;
+
+sub new
+{
+    my ($class, $param_hr) = @_;
+
+    die "$class requires 'chokepoints' parameter"
+        if not exists $param_hr->{chokepoints};
+
+    return bless $param_hr, $class;
+}
+
+sub eye
+{
+    my ($self) = @_;
+
+    croak 'no chokepoints encountered'
+        if not @{ $self->{chokepoints} };
+
+    my ($eye_hr)
+        = sort { $a->{cents} <=> $b->{cents} } @{ $self->{chokepoints} };
+
+    return Finance::Budget::Chokepoint->new(
+        {   point  => $eye_hr,
+            markup => $self->{markup},
+        }
+    );
+}
+
+sub next
+{
+    my ($self) = @_;
+
+    return
+        if not @{ $self->{chokepoints} };
+
+    my $point_hr = shift @{ $self->{chokepoints} };
+
+    return Finance::Budget::Chokepoint->new(
+        {   point  => $point_hr,
+            markup => $self->{markup},
+        }
+    );
+}
+
+package Finance::Budget::Chokepoint;
+
+use strict;
+use warnings;
+use overload (
+    q{""} => \&_stringify,
+);
+
+sub new
+{
+    my ( $class, $param_hr ) = @_;
+    $param_hr->{point}->{string}
+        = sprintf '%s %s', @{ $param_hr->{point} }{qw( date_str balance_str)};
+    return bless $param_hr, $class;
+}
+
+sub _stringify
+{
+    my ($self) = @_;
+    return $self->{markup}->( $self->{point} );
+}
+
+
 1;
 
 __END__
@@ -432,11 +565,26 @@ budget.
       }
   );
 
+  printf " Opening Balance: % 28s\n",
+      $budget->opening_balance();
+
   while ( my $transaction = $budget->next() )
   {
       print "$transaction\n";
   }
 
+  my $chokepoints = $budget->get_chokepoints();
+
+  printf "\n Eye of the needle:\n%s\n\n",
+      $chokepoints->eye();
+
+  printf " Chokepoints:\n";
+
+  while ( my $point = $chokepoints->next() )
+  {
+      print "$point\n";
+  }
+
 
 =head1 DESCRIPTION
 
@@ -450,6 +598,8 @@ can also be handy in fine tuning your budget to ensure that it is sustainable.
 
 Check out budget.pl in the demo directory of this project.
 
+=head1 CONSTRUCTOR
+
 =head2 PARAMETERS
 
 =over
@@ -543,11 +693,60 @@ this category.
 
 =back
 
-This field also accepts a data filename which will be read using the Perl do()
-function.
+=back
+
+=head1 METHODS
+
+=head2 opening_balance()
+
+Returns a stringified version of the opening balance (supplied in the
+constructor).
+
+=head2 next()
+
+Get the next transaction.
+
+=head2 get_chokepoints()
+
+Returns a chokepoint iterator.
+
+
+=head1 TRANSACTIONS
+
+Each transaction represents one event in your budget lifespan.
+
+You'll probably just be satisified to print each "$transaction" in its
+stringified form, which uses the markup callback you provide.
+
+The transaction object also offers some getter methods.
+
+=over
+
+=item get_date()
+
+=item get_category()
+
+=item get_amount()
+
+=item get_balance()
 
 =back
 
+=head1 CHOKEPOINTS
+
+The chokepoints are the balance minima between the significant expenses and
+the significant income events. You might be interested in iterating through
+these to see where you're going to run into trouble.
+
+=head2 eye()
+
+This is the "eye" as in threading the needle -- the smallest of the
+chokepoints.
+
+=head2 next()
+
+Get the next chokepoint.
+
 
 =head1 SEE ALSO
 
